'''
Created on Jun 27, 2017

@author: patitz
'''

import fnmatch
import copy

# It is assumed that the gadget being added is attaching via its (1,2) side in the lists below
# and patterns will be rotated and reflected as necessary to match
PatternList = [
# Pattern         Changes (format: list of lists, each with [nbr, [add list], [remove list]]
["______",        [ [0, [(1,2),(2,3),(3,4),(4,5),(5,0),(0,6),(6,1)], [] ] ]],

["S_____",        [ [0, [(2,3),(3,4),(4,5),(5,6),(6,0),(0,1)], []], [1, [], [(4,5)] ] ]],
["V_____",        [ [0, [(1,2),(2,3),(3,4),(4,5),(5,0),(0,6)], []], [1, [(2,0),(0,3)], [(3,4),(4,0),(0,5)] ] ]],

["S?____",        [ [0, [(2,0),(0,4),(4,5),(5,6),(6,1)], []], [1, [], [(4,5)] ]]],
["V?____",        [ [0, [(2,0),(0,4),(4,5),(5,6),(6,1)], []], [1, [(5,0),(0,6)], [(4,0),(0,5),(5,6)] ] ]],

["S_?___",        [ [0, [(2,0),(0,5),(5,6),(6,1)], []], [1, [], [(4,5)] ] ]],
["V_?___",        [ [0, [(2,0),(0,5),(5,6),(6,1)], []], [1, [(5,0),(0,6)], [(4,0),(0,5),(5,6)] ] ]],

["S__?__",        [ [0, [(2,3),(3,0),(0,6),(6,1)], []], [1, [], [(4,5)] ] ]],
["V__?__",        [ [0, [(2,3),(3,0),(0,6),(6,1)], []], [1, [(5,0),(0,6)], [(4,0),(0,5),(5,6)] ] ]],

["S___??",        [ [0, [(2,3),(3,4),(4,0),(0,1)], []], [1, [], [(4,5)] ] ]],
["V___??",        [ [0, [(2,3),(3,4),(4,0),(0,1)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["S?___?",        [ [0, [(2,3),(3,4),(4,5),(5,0),(0,1)], []], [1, [], [(4,5)] ], [2, [(4,0)], [(4,5),(5,0)] ] ]],
["V?___?",        [ [0, [(2,3),(3,4),(4,5),(5,0),(0,1)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ], [2, [(4,0)], [(4,5),(5,0),(6,1)] ] ]],

["S__??_",        [ [0, [(2,3),(3,0),(0,1)], []], [1, [], [(4,5)] ]]],
["V__??_",        [ [0, [(2,3),(3,0),(0,1)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["S?_?__",        [ [0, [(2,0),(0,6),(6,1)], []], [1, [], [(4,5)] ]]],
["V?_?__",        [ [0, [(2,0),(0,6),(6,1)], []], [1, [(5,0),(0,6)], [(4,0),(0,5),(5,6)] ] ]],

["S_?_?_",        [ [0, [(1,0),(0,2)], []], [1, [], [(4,5)] ] ]],
["V_?_?_",        [ [0, [(1,0),(0,2)], []], [1, [(5,0),(0,6)], [(4,0),(0,5),(5,6)] ] ]],

["S__???",        [ [0, [(2,3),(3,0),(0,1)], []], [1, [], [(4,5)] ] ]],
["V__???",        [ [0, [(2,3),(3,0),(0,1)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["S?__??",        [ [0, [(2,3),(3,4),(4,0),(0,1)], []], [1, [], [(4,5)]], [2, [(4,0)], [(4,5),(5,0)] ] ]],
["V?__??",        [ [0, [(2,3),(3,4),(4,0),(0,1)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)]], [2, [(4,0)], [(4,5),(5,0),(6,1)] ] ]],

["S_??_?",        [ [0, [(1,0),(0,2)], []], [1, [], [(4,5)] ] ]],
["V_??_?",        [ [0, [(1,0),(0,2)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["S_???_",        [ [0, [(1,0),(0,2)], []], [1, [], [(4,5)] ] ]],
["V_???_",        [ [0, [(1,0),(0,2)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["S_?_??",        [ [0, [(1,0),(0,2)], []], [1, [], [(4,5)] ] ]],
["V_?_??",        [ [0, [(1,0),(0,2)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["S?_?_?",        [ [0, [(1,0),(0,2)], []], [1, [], [(4,5)] ] ]],
["V?_?_?",        [ [0, [(1,0),(0,2)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["S_????",        [ [0, [(1,0),(0,2)], []], [1, [], [(4,5)] ] ]],
["V_????",        [ [0, [(1,0),(0,2)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["V?_???",        [ [0, [(1,0),(0,2)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["V??_??",        [ [0, [(1,0),(0,2)], []], [1, [(3,0),(0,4)], [(3,4),(4,0),(0,5)] ] ]],

["S?????",        [ [0, [(1,0),(0,2)], []], [1, [], [(4,5)] ] ]],
["V?????",        [ [0, [(1,0),(0,2)], []], [1, [], [] ] ]]
]

# Rotates the characters of a string 'forward' by n spaces (with wraparound)
def rotate(s, n):
    n = -n
    return s[n:] + s[:n]

# Reflects (i.e. reverses) a string
def reflect(s):
    s = s[0] + s[5] + s[4] + s[3] + s[2] + s[1]

    return s

# Swaps the vertex numbers in a list to correspond to the edges being reflected
# across the vertical center line (through the centers of (1,2) and (4,5))
def reflectVerts(l):
    #l = [6 if x==1 else (1 if x==6 else x) for x in l]
    #l = [(1,y) if x==6 else (x,y) for (x,y) in l]
    l = [(1,y) if x==2 else ((2,y) if x==1 else (x,y)) for (x,y) in l]
    l = [(3,y) if x==6 else ((6,y) if x==3 else (x,y)) for (x,y) in l]
    l = [(4,y) if x==5 else ((5,y) if x==4 else (x,y)) for (x,y) in l]
    l = [(x,1) if y==2 else ((x,2) if y==1 else (x,y)) for (x,y) in l]
    l = [(x,3) if y==6 else ((x,6) if y==3 else (x,y)) for (x,y) in l]
    l = [(x,4) if y==5 else ((x,5) if y==4 else (x,y)) for (x,y) in l]
    return l

def MatchPattern(nbrPattern):
    nbrPatternStr = ""
    for n in nbrPattern:
        nbrPatternStr = nbrPatternStr + n
    #print("In MatchPattern, searching for " + nbrPatternStr)
    if not "_" in nbrPatternStr:
        print("ERROR: Need to hardcode handling of situation with all 6 neighbors and no straight edges!!!")
    for entry in PatternList:
        entryCopy = copy.deepcopy(entry)
        pattern = entryCopy[0]
        for i in range(6):
            # try all rotations of nbrPattern
            rotatedNbrPattern = rotate(nbrPatternStr, i)
            #print("checking rotated " + rotatedNbrPattern + " against " + pattern)
            ret = fnmatch.filter([rotatedNbrPattern], pattern)
            if len(ret) > 0:
                rot = -i
                #print("Matched pattern " + str(entryCopy) + " at rotation " + str(rot))
                return (pattern, rot, False, entryCopy[1])

        reflectedNbrPattern = reflect(nbrPatternStr)
        #print("Trying reflection " + str(reflectedNbrPattern))
        for i in range(6):
            rotatedAndReflectedNbrPattern = rotate(reflectedNbrPattern, i)
            #print("checking rotated and reflected " + rotatedAndReflectedNbrPattern + " against " + pattern)
            ret = fnmatch.filter([rotatedAndReflectedNbrPattern], pattern)
            if len(ret) > 0:
                rot = i
                #print("Matched (reflected) pattern " + str(entryCopy) + " at rotation " + str(rot))
                changes = entryCopy[1]
                #print("before reflection: " + str(changes[0][1]))
                changes[0][1] = reflectVerts(changes[0][1])
                #print("after reflection: " + str(changes[0][1]))
                return (pattern, rot, True, entryCopy[1])
            
    return (None, 0, False, [])
