import wx


DEBUG_MSG = 0
MSG_MSG   = 1
IMP_MSG   = 2
WARN_MSG  = 3
ERROR_MSG = 4


def __init__(app):
    global DEBUG_MSG_LEVEL
    global globalApp
    global DocumentationFileList
    
    DEBUG_MSG_LEVEL = WARN_MSG
    globalApp = app
    DocumentationFileList = []


def PrintDebug(msg, level):
    if DEBUG_MSG_LEVEL <= level:
        print msg


SIMU_ID_OK_CHECK   = 0
SIMU_ID_OK_NOCHECK = 1
SIMU_ID_CANCEL     = 2

#----------------------------------------------------------------------------
class DontShowAgainDialog(wx.Dialog):
    def __init__(
            self, parent, ID, title, message, size=wx.DefaultSize, pos=wx.DefaultPosition, style=wx.DEFAULT_DIALOG_STYLE):

        self.bShowAgain = True
        wx.Dialog.__init__(self, parent, ID, title, pos, size, style)
        sizer = wx.BoxSizer(wx.VERTICAL)

        label = wx.StaticText(self, -1, message)
        sizer.Add(label, 0, wx.ALIGN_CENTRE|wx.ALL, 5)

        self.checkbox = wx.CheckBox(self, -1, "Do not show this warning again")
        sizer.Add(self.checkbox, 0, wx.ALIGN_RIGHT|wx.ALL, 5)

        btnsizer = wx.StdDialogButtonSizer()
        
        self.okBtn = wx.Button(self, wx.ID_OK, "OK")
        self.okBtn.Bind(wx.EVT_MOUSE_EVENTS, self.OnDlgButton)
        self.okBtn.SetDefault()
        btnsizer.AddButton(self.okBtn)
        
        self.cancelBtn = wx.Button(self, wx.ID_CANCEL, "Cancel")
        self.cancelBtn.Bind(wx.EVT_MOUSE_EVENTS, self.OnDlgButton)
        btnsizer.AddButton(self.cancelBtn)

        btnsizer.Realize()

        sizer.Add(btnsizer, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT|wx.ALL, 5)

        self.SetSizer(sizer)
        sizer.Fit(self)
        
    def OnDlgButton(self, event):
        if event.LeftDown():
            bChecked = self.checkbox.GetValue()
            if event.EventObject == self.okBtn:
                if bChecked == True:
                    self.EndModal(SIMU_ID_OK_CHECK)
                else:
                    self.EndModal(SIMU_ID_OK_NOCHECK)
            if event.EventObject == self.cancelBtn:
                self.EndModal(SIMU_ID_CANCEL)
        
#----------------------------------------------------------------------------


# The following should be the list of files used by the DocumentationTB
#DocumentationFileList = [ "Overview", "System Parameters", "File Formats", "Tips and Tricks"]

class DocumentationTB(wx.Treebook):
    def __init__(self, parent, idIn):
        wx.Treebook.__init__(self, parent, idIn, style=
                             wx.BK_DEFAULT
                             #wx.BK_TOP
                             #wx.BK_BOTTOM
                             #wx.BK_LEFT
                             #wx.BK_RIGHT
                            )
        
        self.windows = []
        # Now make a bunch of panels for the list book
        for fileName in DocumentationFileList:
            win = self.MakeDocPanel(fileName)
            self.AddPage(win, fileName)
            self.windows.append(win)
            #win = self.MakeDocPanel(colour)
            #self.AddSubPage(win, 'a sub-page')

        # This is a workaround for a sizing bug on Mac...
        #wx.FutureCall(100, self.AdjustSize)

        self.Bind(wx.EVT_TREEBOOK_PAGE_CHANGING, self.OnPageChanging)

    def AdjustSize(self):
        #print self.GetTreeCtrl().GetBestSize()
        self.GetTreeCtrl().InvalidateBestSize()
        self.SendSizeEvent()
        #print self.GetTreeCtrl().GetBestSize()
        

    def MakeDocPanel(self, fileName):
        p = wx.html.HtmlWindow(self) 
        if "gtk2" in wx.PlatformInfo: 
            p.SetStandardFonts() 
        p.LoadPage("docs/" + fileName + ".html")
        return p

    def OnPageChanging(self, event):
        selected = event.GetSelection()
        #print("new: " + str(selected))
        correctDoc = DocumentationFileList[selected]
        self.windows[selected].LoadPage("docs/" + correctDoc + ".html")
        event.Skip()



class SomeCustomDialog(wx.Dialog):
    def __init__(
            self, parent, ID, size=wx.DefaultSize, pos=wx.DefaultPosition, 
            style=wx.DEFAULT_DIALOG_STYLE,
            useMetal=False,
            ):

        title = "Custom dialog"
        
#MJP: This functionality seems to have disappeared or changed in wx phoenix
#===============================================================================
#         # Instead of calling wx.Dialog.__init__ we precreate the dialog
#         # so we can set an extra style that must be set before
#         # creation, and then we create the GUI object using the Create
#         # method.
#         pre = wx.PreDialog()
#         pre.SetExtraStyle(wx.DIALOG_EX_CONTEXTHELP)
#         pre.Create(parent, ID, title, pos, size, style)
# 
#         # This next step is the most important, it turns this Python
#         # object into the real wrapper of the dialog (instead of pre)
#         # as far as the wxPython extension is concerned.
#         self.PostCreate(pre)
#===============================================================================
#MJP: so just replacing with the standard constructor for now since this dialog type isn't currently used
        wx.Dialog.__init__(self, parent, ID, title, pos, size, style)

        # This extra style can be set after the UI object has been created.
        if 'wxMac' in wx.PlatformInfo and useMetal:
            self.SetExtraStyle(wx.DIALOG_EX_METAL)


        # Now continue with the normal construction of the dialog
        # contents
        sizer = wx.BoxSizer(wx.VERTICAL)

        label = wx.StaticText(self, -1, "This is a wx.Dialog")
        label.SetHelpText("This is the help text for the label")
        sizer.Add(label, 0, wx.ALIGN_CENTRE|wx.ALL, 5)

        box = wx.BoxSizer(wx.HORIZONTAL)

        label = wx.StaticText(self, -1, "Field #1:")
        label.SetHelpText("This is the help text for the label")
        box.Add(label, 0, wx.ALIGN_CENTRE|wx.ALL, 5)

        text = wx.TextCtrl(self, -1, "", size=(80,-1))
        text.SetHelpText("Here's some help text for field #1")
        box.Add(text, 1, wx.ALIGN_CENTRE|wx.ALL, 5)

        sizer.Add(box, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        box = wx.BoxSizer(wx.HORIZONTAL)

        label = wx.StaticText(self, -1, "Field #2:")
        label.SetHelpText("This is the help text for the label")
        box.Add(label, 0, wx.ALIGN_CENTRE|wx.ALL, 5)

        text = wx.TextCtrl(self, -1, "", size=(80,-1))
        text.SetHelpText("Here's some help text for field #2")
        box.Add(text, 1, wx.ALIGN_CENTRE|wx.ALL, 5)

        sizer.Add(box, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        line = wx.StaticLine(self, -1, size=(20,-1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.RIGHT|wx.TOP, 5)

        btnsizer = wx.StdDialogButtonSizer()
        
        if wx.Platform != "__WXMSW__":
            btn = wx.ContextHelpButton(self)
            btnsizer.AddButton(btn)
        
        btn = wx.Button(self, wx.ID_OK)
        btn.SetHelpText("The OK button completes the dialog")
        btn.SetDefault()
        btnsizer.AddButton(btn)

        btn = wx.Button(self, wx.ID_CANCEL)
        btn.SetHelpText("The Cancel button cancels the dialog. (Cool, huh?)")
        btnsizer.AddButton(btn)
        btnsizer.Realize()

        sizer.Add(btnsizer, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        self.SetSizer(sizer)
        sizer.Fit(self)

#----------------------------------------------------------------------------

class DocumentationFrame(wx.Frame):
    def __init__(self, app):
        wx.Frame.__init__(self, None, -1, "Oritatami Simulator Documentation", pos=(50,50), size=(400,400),
                         style=wx.DEFAULT_FRAME_STYLE, name="Oritatami Simulator Documentation")
        self.app = app
        self._mgr = wx.aui.AuiManager(self)
        self.SetIcon(wx.Icon(wx.Bitmap("images/logo.png", wx.BITMAP_TYPE_PNG)))

        self.treewin = DocumentationTB(self, -1)
        self._mgr.AddPane(self.treewin, wx.CENTER, 'Documentation Window')
        
        self.Bind(wx.EVT_CLOSE, self.OnCloseDocFrame)
        self.CreateStatusBar(3)
        self.SetSize((1200, 800))
        self.SetFocus()

        self._mgr.Update()
        #self.Maximize()
        self.Show(True)
        
    def OnCloseDocFrame(self, evt):
        # deinitialize the frame manager
        self._mgr.UnInit()
        self.app.docFrame = None
        # delete the frame
        self.Destroy()

